Data formats
============

Python can handle many data formats "out of the box" using its standard
library.  How to read/write CSV and XML are illustrated, as well as how
to read binary data generated by, e.g., C programs.

What is it?
-----------
1. Binary data
  * `write_doubles.c`: C code to write a sequence of `double` to a binary
    file (square roots of integers).
  * `read_doubles.c`: C code to read a binary file containing a sequence
    of `double`, and print those in ASCII representation to verify the
    contents of the binary file.
  * `Makefile`: to build the C programs.
  * `read_doubles.py`: reads sequences of 8 bytes, unpacks them into
    a Python variable, and prints them in ASCII to standard out.
  * `doubles.bin`: binary file.
1. CSV files
  * `write_csv.py`: uses the standard library `csv` module to create
    a CSV file with four columns and five rows.
  * `read_csv.py`: reads a CSV file (e.g., `data.csv`) that has two
    columns, `name` and `weight` and prints the values to standard output.
    It uses the CSV `sniffer()` function to detect the CSV dialect.
  * `data.csv`: example file to use with `read_csv.py`.
  * `read_commented_csv.py`: illustrates some fiddling with files that
    are not truly CSV snice they have a comment header
  * `data_commented_tabs.csv`: tab separated CSV ile
  * `data_commented_commas.csv`: comma separated CSV ile
  * `data_commented_semicolon.csv`: semicolon separated CSV ile
  * `read_csv_rows.py`: illustrates the default CSV reader
  * `agt_parser.py`: parser for a file format that is partially CSV.
  * `agt_data`: data files to parse using `agt_parser.py`.
1. XML files
  * `write_xml.py`: creates XML that has a root-level `blocks` element,
    containing `block` elements that are named (by attribue), and can
    have `item` elements, where the latter contain a text element.
    Use the `-h` options to see how to specify parameters.
    XML is generated using the `xml.minidom` module in the standard
    library.
  * `read_xml.py`: reads an XML file in the format described above,
    and writes each item, preceeded by its block's name to standardi
    output.  However, this program can also deal with nested blocks, i.e.,
    an XML file where a `block` element can contain another `block`
    element.
    The SAX parser in the `xml.sax` module is used for parsing the XML.
    A `ContentHandler` is implemented, and a `Block` class is used for
    data representation.
  * `blocks.xml`: example XML file.
  * `nested_blocks.xml`: example XML file containng nested block elements.
1. Text as binary
  * `line_indexer.py`: indexes a text file, i.e., it produces a CSV file
    with two columns, the first the file position of the start of each line
    in the text file, the second the length of that line, line endings
    exclusive.  The text file is read in binary mode.
  * `index.txt`: example file to index
  * `read_line_index.py`: test script that takes a text file as input,
    a file position and a line length, and prints the characters read
    to standard output for verification, quoted by '|'.
1. CSV, HDF5 and JSON combination

  For more (and better examples of reading and writing, see
  the [DataStorage](https://github.com/gjbex/training-material/tree/master/DataStorage/Hdf5) section.
  * `data_generator.py`: this script will generate numerical data (integer
    and floating point) using specified random number distributions.  The
    column names, types the distributions, and the parameters they require
    are specified in a JSON file that is read.  The generated data is
    written to a file in either CSV or HDF5 format.
  * `mixed_data.json`: example JSON definition file for the data table to
    be generated.


## Other formats

Example code for using NetCDF can be found in the [DataFormats](https://github.com/gjbex/training-material/tree/master/DataStorage/NetCDF) section.
