Integrating C, C++ and Fortran code with Python
===============================================

Two ways to integrate C/C++ code are illustrated, using `ctype` and
SWIG.  For Fortran code, `f2py` is used.

What is it?
-----------
1. Fortran
  * `array_utils.f90`: a Fortran module that defines a function to
    sum the elements of a double precesion array.  `f2py` creates
    a shared library and a wrapper file.
  * `array_utils_f2py.f90`: loads the module defined in the wrapper
    file and calls the `sum` function with a `numpy` array.
  * `sum.f90`: Fortran program to test `array_utils.f90`
  * `pi.f90`: Fortran function to compute pi from random numbers, `f2py`
    is used to create a shared library and a wrapper.
  * `comp_pi_f2py.py`: test program to compute pi using a given number
    of iterations.
2. C
  * `logistic_map.[ch]`: function that computes the logistic map for a
    given number of steps, an initial value, and a factor.  This is
    compiled into a shared library, or, using `logistic_map.i` and
    `SWIG` to generate a wrapper, into `_log_map.so` shared library.
  * `log_map_cdll.py`: loads a simple shared library, retrieves the
    functions, and calls it.
  * `logistic_map.i`: SWIG file describing the function in `logistic_map.c`
    so that a C wrapper can be generated using `swig` and together with
    the C file to produce a shared library.  Functions can be directly
    imported from that library using the Python wrapper generated by
    `swig`.
  * `main.c`: C driver for the logistic map.
  * `params.txt`: parameter-weaver file describing the command line
    arguments for `main.c`.
  * `point.[chi]`: C implementation of a point as a structure, and SWIG
    declaration file.
  * `point_swig.py`: uses `point.c` generated module.
  * `point_class.[cxx,h,i]`: a C++ class defining point objects, and
    corresponding SWIG declaration file.
  * `point_class_swig.py`: uses `point_class.cxx` generated module.
  * `stats.[chi]`: defines a function that takes an array of `double`
    as argument, and returns a structure, with corresponding SWIG
    declaration file.
  * `stats_swig.py`: uses `stats.c` generated module.
  * `python-interfacing.pptx`: Powerpoint presentation introducing
    interfacing with C, C++, and Fortran
  
