Visualization of CSV files
==========================

CSV (Comma-Separted Values) is a convenient, although inefficient data
format.  It has a number of advantages:
1. it is trivial to read/write from any programming language;
2. most data processing packages can import it without problems;
3. since the representation is textual, it can be inspected using any
   editor.
However, it has a number of drawbacks as well:
1. it is inefficient with respect to file size;
2. one can not be sure that the string representation of a floating
   point number will result in the same floating point number when
   reread (i.e., that it will have the same bit representation when
   it is input, as it had when it was output).

Examples
--------
This directory contains a number of example data files (.csv), as well
as some ParaView state files (.pvsm) that illustrate how they can be
imported and vidualized in ParaView.

1. Particle data
  * `particles.csv`: particles have a postion in 3D space,
    a velocity (represented by its x-, y-, and z-components), a mass and a
    charge.
  * `particles.pvsm`: it was rendered by representing the particles as
    spheres, with a radius determined from their mass, assuming uniform
    density of `1 kg/m^3` (`radius = ((3*mass)/(4*pi))^(1/3)`), colored by
    charge.  The velocity is computed from its components, and rendered as
    arrow glyph with a length proportional to their magnitude, and colored
    accordingly.
2. Scalar field
  * `scalar_field.csv`: the value of a field is computed on each point
    of a cubic grid (10 x 10 x 10) using the function
    `exp(-(x^2 + y^2 + z^2))`.  The file contains the three coordinates and
    the computed value.
  * `scalar_field.pvsm`: the tabel data is converted to a structured grid,
    and the value is visualized as a surface colored according to value.
    A clip in the x-, y-, and z-direction has been performed to reveal the
    origin.
  * `scalar_field.pdf`: exported image of the end result of the
    visualization process.
  * `scalar_field_trace.py`: trace file generated during the visualization
    process that can be used with pvbatch to reproduce the rendering as a
    JPEG file.  See notes below for details.
  * `scalar_field.jpeg`: image produced by running `scalar_field_trace.py`
    with pvbatch.
  * `scalar_field.py`: Python script to create the data set.  Options are
    available to control the size of the grid, as well as the function by
    specifying scaling factors along the x-, y-, and z-direction.
3. Vector field
  * `vector_field.csv`: velocity represented as x-, y-, and z-components
    as well as the x-, y-, and z-cooridnates of the grid points they where
    computed on.
  * `vector_field.pvsm`: the grid coordinates are converted to points,
    and the velocity vector computed from its components.  The velocity
    is rendered as arrow glyphs with length proportional the velocity, and
    colored accordingly.
  * `vector_field.py`: Python script to generate the CSV data.

Notes
-----
* The ParaView state files were generated by ParaView 4.1 and may not work
with other versions.
* The ParaView trace file was generated using the Windows version of
  ParaView and had to be modified slightly to work with a Linux pvbatch.
  1. File paths (input/output) are absolute, and in Windows format, so
     they were changed to be relative to the working directory, and Linux
     style.
  2. Attributes 'Row format' and 'Point format' are unknown to Linux
     pvbatch, and the corresponding lines have simply been commented out.
  3. For convenience, a command line optons parser (argparse) was added
     to the trace file so that the input and output file name can be
     specified on the command line using the `-i` and `-o` flags
     respectively, .e.g.,

     ```bash
     $ pvbatch scalar_field_trace.py -i scalar_field.csv -o rendered.jpeg
     ```

